use crate::helpers::db;
use crate::schema::{artists, artists_tracks};
use chrono::NaiveDateTime;
use diesel::result::Error;
use diesel::{
    AsChangeset, ExpressionMethods, Insertable, QueryDsl, Queryable, RunQueryDsl, Selectable,
};
use serde::{Deserialize, Serialize};

#[derive(AsChangeset, Insertable, Queryable, Selectable, Deserialize, Serialize)]
#[diesel(table_name = crate::schema::artists)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct Artist {
    pub id: String,
    pub name: String,
    pub slug: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

#[derive(Debug, Deserialize, Queryable, Serialize)]
pub struct Artists {
    pub id: String,
    pub title: String,
    pub slug: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

impl Artists {
    pub fn find(id: String) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = artists::table.filter(artists::id.eq(id)).first(conn)?;
        Ok(playlist)
    }

    pub fn create(artist: Artist) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(artists::table)
            .values(Artist::from(artist))
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn find_by_track(track_id: &str) -> Result<Vec<Artists>, Error> {
        let conn = &mut db::connection()?;
        let tracks: Vec<(String, String)> = artists_tracks::table
            .filter(artists_tracks::track_id.eq(track_id))
            .get_results::<(String, String)>(conn)?;

        let artists = tracks
            .into_iter()
            .map(|(artist_id, _)| Artists::find(artist_id).unwrap())
            .collect::<Vec<Artists>>();

        Ok(artists)
    }
}

impl Artist {
    fn from(artist: Artist) -> Artist {
        Artist {
            id: artist.id,
            name: artist.name,
            slug: artist.slug,

            created_at: artist.created_at,
            updated_at: artist.updated_at,

            spotify_id: artist.spotify_id,
            tidal_id: artist.tidal_id,
        }
    }
}
