use crate::helpers::db;
use crate::models::tracks::{Tracks, TracksWithArtists};
use crate::models::user::Users;
use crate::schema::playlists;
use chrono::NaiveDateTime;
use diesel::result::Error;
use diesel::{
    AsChangeset, ExpressionMethods, Insertable, QueryDsl, Queryable, RunQueryDsl, Selectable,
};
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct PlaylistCreator {
    pub id: String,
    pub name: String,
}

#[derive(AsChangeset, Insertable, Queryable, Selectable, Deserialize, Serialize)]
#[diesel(table_name = crate::schema::playlists)]
#[diesel(belongs_to(Users))]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct Playlist {
    pub id: String,
    pub name: String,
    pub public: bool,
    pub creator_id: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,
}

#[derive(Debug, Deserialize, Queryable, Serialize)]
pub struct Playlists {
    pub id: String,
    pub name: String,
    pub public: bool,
    pub creator_id: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,
}

impl Playlists {
    pub fn find(id: &str) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = playlists::table.filter(playlists::id.eq(id)).first(conn)?;
        Ok(playlist)
    }

    pub fn create(playlist: NewPlaylist) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(playlists::table)
            .values(playlist)
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn find_for_user(user_id: &str, filter_public: bool) -> Result<Vec<Playlists>, Error> {
        let conn = &mut db::connection()?;

        let mut playlists = playlists::table
            .filter(playlists::creator_id.eq(user_id))
            .into_boxed();

        if filter_public {
            playlists = playlists.filter(playlists::public.eq(true));
        }

        let playlists = playlists.get_results(conn)?;
        Ok(playlists)
    }

    pub fn get_tracks(&self) -> Result<Vec<TracksWithArtists>, Error> {
        let tracks = Tracks::find_by_playlist(&self.id)?;
        let tracks: Vec<TracksWithArtists> = tracks
            .into_iter()
            .map(Tracks::with_artists)
            .collect::<Result<Vec<TracksWithArtists>, _>>()?;

        Ok(tracks)
    }

    pub fn get_creator(&self) -> Result<Users, Error> {
        let creator = Users::find(&self.creator_id)?;
        Ok(creator)
    }
}

#[derive(Debug, Insertable, Deserialize)]
#[diesel(table_name = playlists)]
pub struct NewPlaylist {
    pub name: String,
    pub public: bool,
    pub creator_id: Option<String>,
}

impl Playlist {
    pub fn create(name: &str, public: bool, creator_id: &str) -> NewPlaylist {
        NewPlaylist {
            name: name.to_string(),
            public,
            creator_id: Some(creator_id.to_string()),
        }
    }

    fn from(playlist: Playlist) -> Playlist {
        Playlist {
            id: playlist.id,
            name: playlist.name,
            public: playlist.public,
            creator_id: playlist.creator_id,

            created_at: playlist.created_at,
            updated_at: playlist.updated_at,
        }
    }
}
